import java.util.Scanner;

public class GameSystemConsole {

    // VARIABLES
    private int numberOfCards = 3;                      //  Sets up the  number of cards we need - This would be better read in from a database - select count[x] from
    private Card[] card = new Card[numberOfCards];      //  Array to hold the cards needed
    private String userName;                            //  String to hold username
    private boolean firstRun = false;

    /**
     *
     *  Runs the methods below
     *  <p>
     *  createCards()
     *  <p>
     *  createUserName()
     *  <p>
     *  startGame()
     *  <p>
     *  newGame()
     *
     */
    public void run() {
        createCards();
        createUserName();
        startGame();
        newGame();
    }

    /**
    *  Creates an array of 'card' objects used to play the game.
    *  <p>
    *  This method relies on the card_n methods being run.
    *  Each 'card method' populates a card object present in the array created here
    *
    *  @see card_0()
    */
    private void createCards() {
        for (int i = 0; i < numberOfCards; i++) {   //  for loop (runs for numberOfCards = n (see Varables declaration at the top))
            card[i] = new Card();                   //  Creates a 'Card' object and places it into the array a count i.
        }

        //  Calls the methods that give each card it's data.
        card_0();
        card_1();
        card_2();
    }

    /**
    *
    *   Prompts the player to create a name.
    *   <p>
    *   Typing "qqq" or "quit" will terminate the
    *   program early. This method will read in
    *   from a text file and display a banner
    *   <p>
    *   If this method is called again it will
    *   not print the banner or request a user
    *   name (see boolean value)
    *
    */
    private void createUserName() {

        if (firstRun == false) {    // this is only relevant if the user wishes to play the game again. (in which case it will skip the following code.)
            firstRun = true;

            TextReader.readAndPrintFile("GameIntro.txt");   //  Using the method found in TextReader, the file GameIntro.txt is read
            //  A multi line string literal would be nicer but for now this will do.
            Scanner userNameInput = new Scanner(System.in); //  Creates a new obect(Scanner) to read player input
            userName = userNameInput.nextLine();            //  Assigns the input text -> userName(String)


            if ("qqq".equals(userName) || "quit".equals(userName)) { // If you type qqq or quit as your username, the game will terminate.
                System.exit(0);
            } else {
                System.out.println(userName + "... A Fine name indeed.");   // Prints out userName(String) to the console.
            }
            System.out.println();   //  New line
        }
    }

    private void userNameHack() {   //Used in testing when typing in a username each time is time consuming
        userName = "Adventurer";
    }

    private String getUserName() {  //Returns the userName as String
        return userName;
    }

    /**
    *
    *   Asks the player if they wish to play again.
    *   <p>
    *   This method will run once the user reaches
    *   a game over card - user asked to input a char
    *   <p>
    *   Should the user wish to continue playing -
    *   they will not be promted to recreate a username,
    *   instead they will be launched back into the game.
    *   <p>
    *   Otherwise they will be broken out of the game loop.
    *
    */
    private void newGame() {

        System.out.println("*****************************\nWould you like to play again? (Y/N)");   //  Prints string to console

        int playerChoice = 1;                           //  creates while loop condition
        while (playerChoice == 1) {                     //  while condition = 1
            Scanner newGame = new Scanner(System.in);   //  get playerInput with Scanner created here
            String playerInput = newGame.nextLine();    //  assign newGame(Scanner input) to playerInput


            //  fixed to support lower JDK versions. (was previously SelectCase 1.7)
            if (("y".equals(playerInput)) || ("Y".equals(playerInput))) {         //  if either 'y' or 'Y' are entered
                run();                                                            //  starts a new game by running 'run()'
                playerChoice = 0;                                                 //  kill loop
            } else if (("n".equals(playerInput)) || ("N".equals(playerInput))) {  //  if either 'n' or 'N' are entered
                System.out.println("Thank you for playing " + userName + ".");    //  Print String + userName
                playerChoice = 0;                                                 //  Kill loop
            } else {                                                              //  ELSE/OTHERWISE
                System.out.println("Please type either Y or N");                  //  Ask player for a valid input and loop runs again
            }
        }
    }


    /**
    *
    *   Starts the game loop.
    *
    */
    private void startGame() {
        Card currentCard = card[0];     //  sets the currently active card to 0 (within the array)
        boolean running = true;         //  The for loop running flag

        while (running == true) {       //  While the flag is true

            if (currentCard == null) {  //  Check to see if the current card is a null card (ie hasn't been written)
                //  NOTE: this is a bug check and shouldn't be needed when all cards are present
                //  TODO: for debugging purposes it might be a good idea to print the 'tree' of cards
                //  accessed to a log file, that way I can go quickly back to where I need to be in case of a bug.
                System.out.println("\n" + "You have reached a null card");  //  Prints out String to console
                System.out.println("GAME OVER");                            //  Prints out String to console
                running = false;                                            //  Ends enclosing while loop
            } else if (currentCard.deathCard == true) {         //  If the current card holds the 'deathCard' flag to be true, then end the game
                System.out.println("\n____" + currentCard.mainStory + "____");  //  Prints out main story for the current card
                System.out.println("\n" + "You have died");     //  Prints out String to console
                System.out.println("GAME OVER");                //  Prints out String to console
                running = false;                                //  Ends enclosing while loop
            } else {    //  OtherWise if the card is clean and can be used to advance the story without killing the player etc
                System.out.println("\n____" + currentCard.mainStory + "____"); //  Prints out main story for the current card

                //  This section prints the userChoices to the screen
                String[] stories = new String[4];           //  Creates an Array of 4 Strings
                stories[0] = currentCard.story1;            //  Assigns each String in the array to equal the current card's story
                stories[1] = currentCard.story2;
                stories[2] = currentCard.story3;
                stories[3] = currentCard.story4;

                for (int i = 0; i < currentCard.choiceIndex; i++) {     //  Loop runs for as long as the current choices available on the card
                    //  Example:
                    //          Card has 2 Stories available -
                    //              Prints out STRING loop counter + 1 for the numbers
                    //              Print out the story in the Array at position i
                    System.out.print((i + 1) + ": ");
                    System.out.println(stories[i] + "\n");
                }

                currentCard = advanceStory(currentCard);  //  To get some user input, we need to run this method
                //  Assign the current card with the returned value from chooseYourFate(currentCard)
            }
        }
    }


    /**
    *   advanceStory
    *
    *   @PARAM currentCard (current card object that a player wishes to branch from)
    *   @RETURN The next card in the users story based on their choice selection (linked from currentCard)
    */
    private Card advanceStory(Card currentCard) {
        Card newCard = currentCard;         //  assigns the current card to a local variable of type card.

        int validInputReached = 0;          //  while loop condition
        while (validInputReached == 0) {    //  until a valid input is reached

            // this block gets userInput
            Scanner scan = new Scanner(System.in);  //  Creates a new Scanner called scan and gets the users input
            int choiceMade = 0;                     //  initializes a new int called choiceMade
            if (scan.hasNextInt()) {                //  if the user input is a number
                choiceMade = scan.nextInt();        //  assign the value to choiceMade
            }

            Card[] cardOptionArray = new Card[4];       //  Create an array of Cards called cardOptionArray
            cardOptionArray[0] = newCard.cardOption1;   //  Assign the array positions to the card options listed on each card.
            cardOptionArray[1] = newCard.cardOption2;
            cardOptionArray[2] = newCard.cardOption3;
            cardOptionArray[3] = newCard.cardOption4;

            //  if (the users choice is greater than 0     AND     less than or equal the number of choices on the card)
            //  So basically should the user select a valid value from the card.
            if ((choiceMade > newCard.choiceIndex - newCard.choiceIndex) && (choiceMade <= newCard.choiceIndex)) {

                newCard = cardOptionArray[choiceMade - 1];      //  Assign newCard with the card in the - ARRAY[choice made -1] <-- array is -1
                validInputReached = 1;                          //  breaks out of the array
            } else {    //  Since the user will not have satisfied the requirements
                        //  They will be asked for a number between ONE and the choice index on the current card
                System.out.println("I need a number from 1-" + newCard.choiceIndex);
            }
        }
        return newCard;    // Once everything checks out, the loop will break and the newCard will be returned
    }


    

    /**
     * ***********************************************************************
     *
     *
     *
     *
     * CARD METHODS GO BELOW HERE - CARDS ARE DEFINED WITH 4 VARS MAIN STORY
     * CHOICEINDEX - the method advance story will check for the number and
     * display the stories depending on this needs tidied up later on mind you.
     *
     *
     *
     *
     ************************************************************************
     */
    private void card_0() {     //  Card Zero
        card[0].mainStory = "You feel the breeze on your face"; //  Cards header story

        card[0].choiceIndex = 2;    //  Choice index - this corresponds to the stories below and allows the user to make 2 choices in this instance.

        //  String Options displayed to the player
        card[0].story1 = "Looking around you decide to follow the sandy path towards the beach";
        card[0].story2 = "Spying a small rock at your feet, you decide to pick it up";

        //  Dependencies for current Card
        card[0].cardOption1 = card[1];
        card[0].cardOption2 = card[2];
    }

    private void card_1() {    //  Card One
        //  Cards header story
        card[1].mainStory = "You have arrived at the beach";

        card[1].choiceIndex = 4;

        //  String Options displayed to the player
        card[1].story1 = "You decide to dig a hole";
        card[1].story2 = "You swim in the sea";
        card[1].story3 = "You decide to shake your bum";
        card[1].story4 = "You decide chill and get a tan";
    }

    private void card_2() {    //  Card Two
        //  Cards header story
        card[2].mainStory = "An evil looking crab rips out your eyes.";

        // no need for a choice index here as the user will not be presented with any choices to make!

        //  Death Card is true
        card[2].deathCard = true;
    }
}

